"""
load_and_clean_data.py
====================================
Script to load the raw data and then clean it.
"""

from tools import raw_loader, cleaner
from utility.data import write_pickle
import paths as pt

def main():
    loader = raw_loader.RawLoader2021()
    ats = loader.load_assistive_aids(pt.PATHS_2021[1],
                                     pt.PATHS_2021[3],
                                     pt.PATHS_2021[5],
                                     pt.RAW_DATA_DIR_2021)
    td = loader.load_training_done(pt.PATHS_2021[2],
                                   pt.PATHS_2021[4],
                                   pt.RAW_DATA_DIR_2021)
    sc = loader.load_screening_content(pt.PATHS_2021[2],
                                       pt.PATHS_2021[4],
                                       pt.RAW_DATA_DIR_2021)
    tc = loader.load_training_cancelled(pt.PATHS_2021[2],
                                        pt.PATHS_2021[4],
                                        pt.RAW_DATA_DIR_2021)
    ss = loader.load_status_set(pt.PATHS_2021[2],
                                pt.PATHS_2021[4],
                                pt.RAW_DATA_DIR_2021)
    ic = loader.load_iso_classes('isoall.txt', pt.REFERENCES_DIR)

    cleaner2021 = cleaner.Cleaner2021()
    patient_data = td[['CitizenId', 'Gender', 'BirthYear']].drop_duplicates(keep='first')
    patient_data = cleaner2021.clean_patient_data(patient_data)
    screening_content = cleaner2021.clean_screening_content(sc, patient_data)
    status_set = cleaner2021.clean_status_set(ss, patient_data)
    training_done = cleaner2021.clean_training_done(td, patient_data)
    training_cancelled = cleaner2021.clean_training_cancelled(tc, patient_data)
    assistive_aids = cleaner2021.clean_assistive_aids(ats, ic)

    write_pickle(screening_content, pt.INTERIM_DATA_DIR, 'sc.pkl')
    write_pickle(status_set, pt.INTERIM_DATA_DIR, 'ss.pkl')
    write_pickle(training_done, pt.INTERIM_DATA_DIR, 'td.pkl')
    write_pickle(training_cancelled, pt.INTERIM_DATA_DIR, 'tc.pkl')
    write_pickle(assistive_aids, pt.INTERIM_DATA_DIR, 'ats.pkl')
    write_pickle(ic, pt.INTERIM_DATA_DIR, 'ic.pkl')

if __name__ == "__main__":
    main()
