"""
make_dataset_count.py
====================================
Script to make a dataset for Complete, Compliance, Fall and Risk cases
where categorial features are encoded as an integer array with
values as a count of the number of times a value appears in a column.
"""

import paths as pt
from tools import preprocessor
from utility import embedder
from utility.config import load_config
from utility.data import read_csv, write_csv
import pandas as pd
import numpy as np

def main():
    for label_name in ["Complete", "Compliance", "Fall", "Risk", "Alarm"]:
        data_settings = load_config(pt.CONFIGS_DIR, 'data.yaml')
        ats_resolution = data_settings['ats_resolution']

        if label_name == "Risk":
            target_settings = load_config(pt.CONFIGS_DIR, f'{label_name.lower()}.yaml')
            ex_resolution = target_settings['ex_resolution']

        if label_name in ["Complete", "Compliance", "Fall", "Alarm"]:
            converters = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
            file_path = pt.PROCESSED_DATA_DIR
            file_name = f'{label_name.lower()}.csv'
            df = read_csv(file_path, file_name, converters=converters)
        else:
            ex = {str(i)+'Ex':str for i in range(1, ex_resolution+1)}
            ats = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
            converters = {**ex, **ats}
            file_path = pt.PROCESSED_DATA_DIR
            file_name = f'{label_name.lower()}.csv'
            df = read_csv(file_path, file_name, converters=converters)

        if label_name in ["Complete", "Compliance", "Fall", "Alarm"]:
            cols_ats = [str(i)+'Ats' for i in range(1, ats_resolution+1)]
            unique_ats = [df[f'{i}Ats'].unique() for i in range(1, ats_resolution+1)]
            unique_ats = list(set(np.concatenate(unique_ats)))
            df_ats = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')
            df_ats = df_ats.drop(['Ats_0'], axis=1)
            df = df.drop(cols_ats, axis=1)
            if label_name == "Alarm":
                df = pd.concat([df.drop("Status", axis=1), df_ats, df[["Status"]]], axis=1)
            else:
                df = pd.concat([df.drop(label_name, axis=1), df_ats, df[[label_name]]], axis=1)
        else:
            cols_ex = [str(i)+'Ex' for i in range(1, ex_resolution+1)]
            unique_ex = [df[f'{i}Ex'].unique() for i in range(1, ex_resolution+1)]
            unique_ex = list(set(np.concatenate(unique_ex)))
            df_ex = preprocessor.extract_cat_count(df, unique_ex, cols_ex, 'Ex_')
            cols_ats = [str(i)+'Ats' for i in range(1, ats_resolution+1)]
            unique_ats = [df[f'{i}Ats'].unique() for i in range(1, ats_resolution+1)]
            unique_ats = list(set(np.concatenate(unique_ats)))
            df_ats = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')
            df = pd.concat([df, df_ex, df_ats], axis=1)
            ex_columns = ['Ex_' + ex for ex in unique_ex]
            ats_columns = ['Ats_' + ats for ats in unique_ats]
            num_cols = embedder.get_numerical_cols(df, label_name)
            df = df[num_cols + ex_columns + ats_columns + [label_name]]
            df = df.drop(['Ex_0', 'Ats_0'], axis=1)

        file_path = pt.PROCESSED_DATA_DIR
        file_name = f'{label_name.lower()}_count.csv'
        write_csv(df, file_path, file_name)

if __name__ == "__main__":
    main()
