"""
make_dataset_full.py
====================================
Script to make a dataset for Complete, Compliance, Fall and Risk
case based on the screenings.
"""

import paths as pt
from tools import labeler, preprocessor
from utility.config import load_config
from utility.data import read_csv, write_csv
import pandas as pd

def main():
    converters = {'CitizenId': str}
    file_path = pt.INTERIM_DATA_DIR
    file_name = 'screenings.csv'
    screenings = read_csv(file_path, file_name, converters=converters)

    data_settings = load_config(pt.CONFIGS_DIR, 'data.yaml')
    ats_resolution = data_settings['ats_resolution']

    df = screenings.copy()
    accum_screenings = labeler.accumulate_screenings(df, data_settings)

    for label_name in ['Complete', 'Compliance', 'Fall', 'Risk']:
        target_settings = load_config(pt.CONFIGS_DIR, f'{label_name.lower()}.yaml')
        features = target_settings['features']

        if label_name == "Risk":
            ex_resolution = target_settings['ex_resolution']
            risk_period_months = target_settings['risk_period_months']

        if label_name == 'Complete':
            df = labeler.make_complete_label(accum_screenings)
        elif label_name == 'Compliance':
            df = labeler.make_compliance_label(accum_screenings)
        elif label_name == 'Fall':
            df = labeler.make_fall_label(accum_screenings)
        else:
            df = labeler.make_risk_label(accum_screenings, risk_period_months)

        df = preprocessor.split_cat_columns(df, col_to_split='Ats',
                                            tag='Ats', resolution=ats_resolution)
        if label_name == "Risk":
            df = preprocessor.split_cat_columns(df, col_to_split='Ex', tag='Ex',
                                                resolution=ex_resolution)

        if label_name in ["Complete", "Compliance", "Fall"]:
            ats_cols = df.filter(regex='Ats', axis=1)
            df = pd.concat([df[features], ats_cols, df[[label_name]]], axis=1)
        else:
            ats_ex_cols = df.filter(regex='Ats|Ex', axis=1)
            df = pd.concat([df[features], ats_ex_cols, df[[label_name]]], axis=1)

        file_path = pt.PROCESSED_DATA_DIR
        file_name = f'{label_name.lower()}.csv'
        write_csv(df, file_path, file_name)

if __name__ == "__main__":
    main()
