"""
make_dataset_ohe.py
====================================
Script to make a dataset for Complete, Compliance, Fall and Risk
case using one hot encoding of categorial features.
"""

import paths as pt
from tools import preprocessor
import pandas as pd
from utility.config import load_config
from utility.data import read_csv, write_csv

def main():
    for label_name in ["Complete", "Compliance", "Fall", "Risk"]:
        data_settings = load_config(pt.CONFIGS_DIR, 'data.yaml')
        ats_resolution = data_settings['ats_resolution']

        if label_name == "Risk":
            target_settings = load_config(pt.CONFIGS_DIR, f'{label_name.lower()}.yaml')
            ex_resolution = target_settings['ex_resolution']

        if label_name in ["Complete", "Compliance", "Fall"]:
            converters = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
            file_path = pt.PROCESSED_DATA_DIR
            file_name = f'{label_name.lower()}.csv'
            df = read_csv(file_path, file_name, converters=converters)
        else:
            ex = {str(i)+'Ex':str for i in range(1, ex_resolution+1)}
            ats = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
            converters = {**ex, **ats}
            file_path = pt.PROCESSED_DATA_DIR
            file_name = f'{label_name.lower()}.csv'
            df = read_csv(file_path, file_name, converters=converters)

        if label_name in ["Complete", "Compliance", "Fall"]:
            ats_cols = [str(i)+'Ats' for i in range(1, ats_resolution+1)]
            df_enc = preprocessor.one_hot_encode(df, ats_cols)
            df = pd.concat([df.drop(ats_cols + [label_name], axis=1),
                            df_enc, df[[label_name]]], axis=1)
        else:
            ex_cols = [str(i)+'Ex' for i in range(1, ex_resolution+1)]
            ats_cols = [str(i)+'Ats' for i in range(1, ats_resolution+1)]
            total_cols = ex_cols + ats_cols
            df_enc = preprocessor.one_hot_encode(df, total_cols)
            df = pd.concat([df.drop(total_cols + [label_name], axis=1),
                            df_enc, df[[label_name]]], axis=1)

        file_path = pt.PROCESSED_DATA_DIR
        file_name = f'{label_name.lower()}_ohe.csv'
        write_csv(df, file_path, file_name)

if __name__ == "__main__":
    main()
