"""
make_dataset_ordinal.py
====================================
Script to make a dataset for Complete, Compliance, Fall and Risk
using ordinal encoding of categorial features.
"""

import paths as pt
import pandas as pd
import numpy as np
from pathlib import Path
from utility.config import load_config
from utility.data import read_csv, write_csv
from sklearn.preprocessing import OrdinalEncoder

def main():
    for label_name in ["Complete", "Compliance", "Fall", "Risk"]:
        data_settings = load_config(pt.CONFIGS_DIR, 'data.yaml')
        ats_resolution = data_settings['ats_resolution']

        if label_name == "Risk":
            target_settings = load_config(pt.CONFIGS_DIR, f'{label_name.lower()}.yaml')
            ex_resolution = target_settings['ex_resolution']

        if label_name in ["Complete", "Compliance", "Fall"]:
            converters = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
            file_path = pt.PROCESSED_DATA_DIR
            file_name = f'{label_name.lower()}.csv'
            df = read_csv(file_path, file_name, converters=converters)
        else:
            ex = {str(i)+'Ex':str for i in range(1, ex_resolution+1)}
            ats = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
            converters = {**ex, **ats}
            file_path = pt.PROCESSED_DATA_DIR
            file_name = f'{label_name.lower()}.csv'
            df = read_csv(file_path, file_name, converters=converters)

        if label_name in ["Complete", "Compliance", "Fall"]:
            ats_cols = [str(i)+'Ats' for i in range(1, ats_resolution+1)]
            oenc = OrdinalEncoder()
            oenc.fit(df[ats_cols].astype(str))
            df_enc = oenc.transform(df[ats_cols].astype(str))
            df_stack = np.column_stack((df.drop(ats_cols + [label_name], axis=1).values,
                                        df_enc, df[[label_name]].values))
            feature_names = df.columns.tolist()
            df = pd.DataFrame(df_stack, columns=feature_names)
        else:
            ex_cols = [str(i)+'Ex' for i in range(1, ex_resolution+1)]
            ats_cols = [str(i)+'Ats' for i in range(1, ats_resolution+1)]
            total_cols = ex_cols + ats_cols
            oenc = OrdinalEncoder()
            oenc.fit(df[total_cols].astype(str))
            df_enc = oenc.transform(df[total_cols].astype(str))
            df_stack = np.column_stack((df.drop(total_cols + [label_name], axis=1).values,
                                        df_enc, df[[label_name]].values))
            feature_names = df.columns.tolist()
            df = pd.DataFrame(df_stack, columns=feature_names)

        file_path = pt.PROCESSED_DATA_DIR
        file_name = f'{label_name.lower()}_ordinal.csv'
        write_csv(df, file_path, file_name)

if __name__ == "__main__":
    main()
