"""
train_alarm_model.py
====================================
Script to train the model for the Alarm case.
"""

import paths as pt
from tools import data_loader, file_writer
from utility.config import load_config
from sksurv.ensemble import GradientBoostingSurvivalAnalysis
from pathlib import Path
from io import BytesIO
import shutil

def main():
    target_settings = load_config(pt.CONFIGS_DIR, "alarm.yaml")
    dl = data_loader.AlarmDataLoader(pt.PROCESSED_DATA_DIR,
                                     "alarm_emb.csv",
                                     target_settings).load_data()
    X, y = dl.get_data()

    model = GradientBoostingSurvivalAnalysis(n_estimators=100,
                                             learning_rate=0.1,
                                             max_depth=14,
                                             loss='coxph',
                                             min_samples_split=4,
                                             max_features='log2',
                                             subsample=0.4,
                                             dropout_rate=0.7,
                                             random_state=0)
    model.fit(X, y)

    file_path = pt.MODELS_DIR
    file_name = 'alarm_gradboost.joblib'
    with open(Path.joinpath(file_path, file_name), 'wb') as fd:
        outfile = BytesIO()
        file_writer.write_joblib(model, outfile)
        outfile.seek(0)
        shutil.copyfileobj(outfile, fd)

if __name__ == '__main__':
    main()
