"""
train_compliance_model.py
====================================
Script to train the model for the Compliance case.
"""

import paths as pt
from tools import file_writer, data_loader
from utility.config import load_config
import xgboost as xgb
from pathlib import Path
from io import BytesIO
import shutil

def main():
    target_settings = load_config(pt.CONFIGS_DIR, "compliance.yaml")
    dl = data_loader.ComplianceDataLoader(pt.PROCESSED_DATA_DIR,
                                          "compliance_emb.csv",
                                          target_settings).load_data()
    X, y = dl.get_data()

    params = {"n_estimators": 400,
              "booster": "gbtree",
              "max_depth": 8,
              "gamma": 0,
              "colsample_bytree": 0.3,
              "min_child_weight": 9,
              "reg_alpha": 18,
              "reg_lambda": 0.9,
              "learning_rate": 0.05,
              "subsample": 1,
              "use_label_encoder": False,
              "eval_metric": "logloss",
              "objective": "binary:logistic",
              "random_state": 0}

    model = xgb.XGBClassifier(**params)
    model.fit(X, y)

    file_path = pt.MODELS_DIR
    file_name = 'compliance_xgboost.joblib'
    with open(Path.joinpath(file_path, file_name), 'wb') as fd:
        outfile = BytesIO()
        file_writer.write_joblib(model, outfile)
        outfile.seek(0)
        shutil.copyfileobj(outfile, fd)

if __name__ == '__main__':
    main()
