"""
test_cleaner.py
====================================
Test module for cleaner
"""

import pandas as pd
import datetime
from tools.cleaner import Cleaner2021

def test_clean_assistive_aids():
    df = pd.DataFrame({
        'DevISOClass': ['120606', '120606', '120606'],
        'LendDate': ['31/05/18', '31/05/18', '31/05/18'],
        'CitizenId': ['1', '2', '3']})
    iso_classes = pd.DataFrame({'DevISOClass': ['120606']})
    res = Cleaner2021().clean_assistive_aids(df, iso_classes)
    assert not res.empty
    assert len(res) == 3

def test_clean_training_cancelled():
    today = datetime.date.today()
    yesterday = today - datetime.timedelta(days=1)
    df_training = pd.DataFrame({
        'CitizenId': ['41', '42'],
        'RatingDate': [yesterday, today]})
    df_patient = pd.DataFrame({'CitizenId': ['42']})
    res = Cleaner2021().clean_training_cancelled(df_training, df_patient)
    assert not res.empty
    assert res.iloc[0]['CitizenId'] == '42'
    assert res.iloc[0]['RatingDate'] == today
    
def test_clean_training_done():
    today = datetime.date.today()
    yesterday = today - datetime.timedelta(days=1)
    df_training = pd.DataFrame({
        'CitizenId': ['41', '42'],
        'RatingDate': [yesterday, today]})
    df_patient = pd.DataFrame({'CitizenId': ['42']})
    res = Cleaner2021().clean_training_done(df_training, df_patient)
    assert not res.empty
    assert res.iloc[0]['CitizenId'] == '42'
    assert res.iloc[0]['RatingDate'] == today
    
def test_clean_status_set():
    today = datetime.date.today()
    yesterday = today - datetime.timedelta(days=1)
    df_training = pd.DataFrame({
        'CitizenId': ['41', '42'],
        'ChangeDate': [yesterday, today]})
    df_patient = pd.DataFrame({'CitizenId': ['42']})
    res = Cleaner2021().clean_status_set(df_training, df_patient)
    assert not res.empty
    assert res.iloc[0]['CitizenId'] == '42'
    assert res.iloc[0]['ChangeDate'] == today

def test_clean_screening_content():
    today = datetime.date.today()
    yesterday = today - datetime.timedelta(days=1)
    df_training = pd.DataFrame({
        'CitizenId': ['41', '42'],
        'ScreeningDate': [yesterday, today]})
    df_patient = pd.DataFrame({'CitizenId': ['42']})
    res = Cleaner2021().clean_screening_content(df_training, df_patient)
    assert not res.empty
    assert res.iloc[0]['CitizenId'] == '42'
    assert res.iloc[0]['ScreeningDate'] == today

def test_clean_patient_data():
    df_patient = pd.DataFrame({'CitizenId': ['42', '0000000000',
                                             '0', '#VALUE!', '681']})
    res = Cleaner2021().clean_patient_data(df_patient)
    assert not res.empty
    assert len(res) == 1
