"""
test_data_loader.py
====================================
Test module for data loader
"""

from tools import data_loader
from utility.config import load_config
import paths as pt
import sklearn.utils as skut

def test_complete_loader():
    settings = load_config(pt.TESTS_FILES_DIR, "test_config.yaml")
    dl = data_loader.CompleteDataLoader(pt.TESTS_FILES_DIR,
                                        "complete_emb.csv", settings).load_data()
    X, y = dl.get_data()
    skut.check_X_y(X, y)

    X, y = dl.prepare_data()
    skut.check_X_y(X, y)

    X_train, X_test, y_train, y_test = dl.prepare_data_split(test_size=0.3)
    skut.check_X_y(X_train, y_train)
    skut.check_X_y(X_test, y_test)

def test_compliance_loader():
    settings = load_config(pt.TESTS_FILES_DIR, "test_config.yaml")
    dl = data_loader.ComplianceDataLoader(pt.TESTS_FILES_DIR,
                                          "compliance_emb.csv", settings).load_data()
    X, y = dl.get_data()
    skut.check_X_y(X, y)

def test_fall_loader():
    settings = load_config(pt.TESTS_FILES_DIR, "test_config.yaml")
    dl = data_loader.FallDataLoader(pt.TESTS_FILES_DIR,
                                    "fall_emb.csv", settings).load_data()
    X, y = dl.get_data()
    skut.check_X_y(X, y)
    
def test_risk_loader():
    settings = load_config(pt.TESTS_FILES_DIR, "test_config.yaml")
    dl = data_loader.RiskDataLoader(pt.TESTS_FILES_DIR,
                                    "risk_emb.csv", settings).load_data()
    X, y = dl.get_data()
    skut.check_X_y(X, y)

def test_alarm_loader():
    settings = load_config(pt.TESTS_FILES_DIR, "test_config.yaml")
    dl = data_loader.AlarmDataLoader(pt.TESTS_FILES_DIR,
                                     "alarm_emb.csv", settings).load_data()
    X, y = dl.get_data()
    skut.check_consistent_length(X, y)
