"""
test_file_reader.py
====================================
Test module for file reader
"""

from tools import file_reader
from io import StringIO, BytesIO
import pandas as pd
import numpy as np
from sklearn.datasets import make_classification
from sklearn.ensemble import RandomForestClassifier
import csv
import pickle
import joblib
from openpyxl import Workbook

def test_read_csv():
    df = pd.DataFrame(data={'col1': [1, 2], 'col2': [3, 4]})
    infile = StringIO()
    df.to_csv(infile, index=False)
    infile.seek(0)
    content = file_reader.read_csv(infile)
    assert content.shape == (2, 2)

def test_read_embedding():
    mapping = {'0': -0.014424355, '043303': -0.06444023}
    infile = StringIO()
    field_names = ['Ats', 'Embedding']
    writer = csv.writer(infile)
    writer.writerow(field_names)
    for key, value in mapping.items():
        writer.writerow([key, value])
    infile.seek(0)
    content = file_reader.read_embedding(infile)
    assert len(content) > 0

def test_read_array():
    arr = np.array([1, 2, 3, 4])
    infile = BytesIO()
    np.save(infile, arr)
    infile.seek(0)
    content = file_reader.read_array(infile)
    assert len(content) > 0

def test_read_pickle():
    car_dict = {"brand": "Ford", "model": "Mustang", "year": 1964}
    infile = BytesIO()
    pickle.dump(car_dict, infile)
    infile.seek(0)
    content = file_reader.read_pickle(infile)
    assert len(content) > 0

def test_read_joblib():
    X, y = make_classification(random_state=0)
    model = RandomForestClassifier().fit(X, y)
    infile = BytesIO()
    joblib.dump(model, infile)
    infile.seek(0)
    content = file_reader.read_joblib(infile)
    assert len(content) > 0

def test_read_excelfile():
    wb = Workbook()
    ws = wb.active
    ws['A1'] = 42
    ws.append([1, 2, 3])
    infile = BytesIO()
    wb.save(infile)
    infile.seek(0)
    content = file_reader.read_excelfile(infile)
    assert len(content) > 0

def test_read_excelfile_sheets():
    wb = Workbook()
    ws = wb.active
    ws['A1'] = 42
    ws.append([1, 2, 3])
    ws2 = wb.create_sheet("Sheet2")
    ws2['A1'] = 42
    ws2.append([1, 2, 3])
    ws3 = wb.create_sheet("Sheet3")
    ws3['A2'] = 42
    ws3.append([1, 2, 3])
    infile = BytesIO()
    wb.save(infile)
    infile.seek(0)
    content = file_reader.read_excelfile_sheets(infile, 3)
    assert len(content) > 0
