"""
test_file_writer.py
====================================
Test module for file writer
"""

from tools import file_writer
from io import StringIO, BytesIO
import pandas as pd
import numpy as np
from sklearn.datasets import make_classification
from sklearn.ensemble import RandomForestClassifier

def test_write_csv():
    df = pd.DataFrame(data={'col1': [1, 2], 'col2': [3, 4]})
    outfile = StringIO()
    file_writer.write_csv(df, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0

def test_write_mapping():
    mapping = {'0': -0.014424355, '043303': -0.06444023}
    outfile = StringIO()
    file_writer.write_embedding(mapping, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0

def test_write_array():
    arr = np.array([1, 2, 3, 4])
    outfile = BytesIO()
    file_writer.write_array(arr, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0

def test_write_pickle():
    car_dict = {"brand": "Ford", "model": "Mustang", "year": 1964}
    outfile = BytesIO()
    file_writer.write_pickle(car_dict, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0

def test_write_joblib():
    X, y = make_classification(random_state=0)
    model = RandomForestClassifier().fit(X, y)
    outfile = BytesIO()
    file_writer.write_joblib(model, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0

def test_write_shap_importance_plot():
    features = ['Cat', 'Dog']
    importances = [0.7, 0.3]
    title = "Cats vs. dogs"
    outfile = BytesIO()
    file_writer.write_shap_importance_plot(features, importances,
                                           title, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0

def test_write_cv_plot():
    means = [[70.942], [70.942], [70.942],
             [70.942], [70.942], [70.942]]
    stds = [[0.015], [0.015], [0.015],
            [0.015], [0.015], [0.015]]
    metric = 'accuracy'
    num_iter = 1
    clf_names = ['KNN', 'SVM', 'LR', 'XGB', 'RF', 'MLP']
    title = 'My title'
    subtitle = 'My subtitle'
    outfile = BytesIO()
    file_writer.write_cv_plot(means, stds, metric, num_iter,
                              clf_names, title, subtitle, outfile)
    outfile.seek(0)
    content = outfile.read()
    assert len(content) > 0
