"""
test_inputter.py
====================================
Test module for inputter
"""

from tools import inputter
from utility import data
import pandas as pd
import numpy as np
import datetime
from typing import NamedTuple

def test_get_birth_year():
    sc = pd.DataFrame({'BirthYear': ['57']})
    res = inputter.get_birth_year(sc)
    assert res == 57

def test_get_gender():
    sc = pd.DataFrame({'Gender': ['FEMALE']})
    res = inputter.get_gender(sc)
    assert res == 0

def test_get_citizen_data():
    sc = pd.DataFrame({'CitizenId': ['455910', '455911']})
    ss = pd.DataFrame({'CitizenId': ['455910', '455911']})
    td = pd.DataFrame({'CitizenId': ['455910', '455911']})
    tc = pd.DataFrame({'CitizenId': ['455910', '455911']})
    ats = pd.DataFrame({'CitizenId': ['455910', '455911']})
    df = data.Data(sc, ss, td, tc, ats)
    res = inputter.get_citizen_data(df, '455910')
    assert res.sc.shape[0] == 1
    assert res.ss.shape[0] == 1
    assert res.td.shape[0] == 1
    assert res.tc.shape[0] == 1
    assert res.ats.shape[0] == 1

def test_get_window_data():
    td = pd.DataFrame({'RatingDate': ['2020-07-10', '2020-07-15']})
    tc = pd.DataFrame({'RatingDate': ['2020-07-10', '2020-07-15']})
    ss = pd.DataFrame({'ChangeDate': ['2020-07-10', '2020-07-15']})
    start_date = '2020-07-09'
    end_date = '2020-07-11'
    tdw, tcw, ssw = inputter.get_screening_data(td, tc, ss,
                                                start_date, end_date)
    assert tdw.shape[0] == 1
    assert tcw.shape[0] == 1
    assert ssw.shape[0] == 1

def test_convert_date_to_datetime():
    string_date = '01-01-2019'
    datetime_date = datetime.datetime(2019, 1, 1)
    date_format = '%d-%m-%Y'
    res = inputter.convert_date_to_datetime(string_date, date_format)
    assert isinstance(res, datetime.date)
    assert res.ctime() == datetime_date.ctime()

def test_get_cancels_week():
    timestamp_date = pd.Timestamp(datetime.datetime(2020,7,10))
    tcw = pd.DataFrame({'RatingDate': [timestamp_date]})
    res = inputter.get_cancels_week(tcw)
    assert res.item() == "28/2020"

def test_get_interval_length():
    start_date = pd.Timestamp(datetime.datetime(2020,7,10))
    end_date = pd.Timestamp(datetime.datetime(2020,7,31))
    res = inputter.get_interval_length(start_date, end_date)
    assert isinstance(res, float)
    assert res == 3

def test_get_n_cancel_week_min():
    cancels = pd.Series(['10/2020', '10/2020', '11/2020', '12/2020'])
    res = inputter.get_n_cancel_week_min(cancels)
    assert res == 1

def test_get_time_between_training_mean():
    cancel_date1 = pd.Timestamp(datetime.datetime(2020,7,10))
    cancel_date2 = pd.Timestamp(datetime.datetime(2020,7,15))
    tcw = pd.DataFrame({'RatingDate': [cancel_date1, cancel_date2]})
    res = inputter.get_time_between_training_mean(tcw)
    assert res == 5

def get_mean_cancels_week():
    n_cancel = 10
    n_weeks = 5
    res = inputter.get_mean_cancels_week(n_cancel, n_weeks)
    assert res == 2

def test_get_n_weeks_without_training():
    n_weeks = 10
    n_weeks_with_trainings = 2
    res = inputter.get_n_weeks_without_training(n_weeks, n_weeks_with_trainings)
    assert res == 8

def test_get_n_weeks_with_training():
    rating_date = pd.Timestamp(datetime.datetime(2020,7,19))
    tdw = pd.DataFrame({'RatingDate': [rating_date]})
    start_date = pd.Timestamp(datetime.datetime(2020,7,12))
    res = inputter.get_n_weeks_with_training(tdw, start_date)
    assert res == 1

def test_get_training_week():
    rating_date = pd.Timestamp(datetime.datetime(2020,7,19))
    tdw = pd.DataFrame({'RatingDate': [rating_date]})
    start_date = pd.Timestamp(datetime.datetime(2020,7,12))
    res = inputter.get_training_week(tdw, start_date)
    assert res.item() == 1

def test_get_n_training_week_min():
    training_per_week = pd.Series([3, 3, 2])
    res = inputter.get_n_training_week_min(training_per_week, 6, 3)
    assert res == 1

def test_get_n_training_week_max():
    training_per_week = pd.Series([3, 3, 2])
    res = inputter.get_n_training_week_max(training_per_week)
    assert res == 2

def test_get_n_training_window():
    rating_date = pd.Timestamp(datetime.datetime(2020,7,19))
    tdw = pd.DataFrame({'RatingDate': [rating_date]})
    res = inputter.get_n_training_window(tdw)
    assert res == 1

def test_get_n_trainin_week():
    n_weeks = 10
    n_training_window = 2
    res = inputter.get_n_training_week(n_weeks, n_training_window)
    assert res == 0.2

def test_get_max_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = inputter.get_max_evaluation(tdw)
    assert res == 3

def test_get_min_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = inputter.get_min_evaluation(tdw)
    assert res == 1

def test_get_std_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = inputter.get_std_evaluation(tdw)
    assert res == 1

def test_get_mean_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = inputter.get_mean_evaluation(tdw)
    assert res == 2

def test_get_needs_reason():
    Screening = NamedTuple('Screening', [('NeedForHelpReason', str)])
    sc1 = Screening(NeedForHelpReason=np.array('Reason'))
    res = inputter.get_needs_reason(sc1)
    assert res == 'Reason'

def test_get_physics_reason():
    Screening = NamedTuple('Screening', [('PhysicalStrengthReason', str)])
    sc1 = Screening(PhysicalStrengthReason=np.array('Reason'))
    res = inputter.get_physics_reason(sc1)
    assert res == 'Reason'
