"""
test_integration.py
====================================
Test module for integration tests
"""

import pytest
import sklearn.utils as skut
import pandas as pd
import paths as pt
from tools import preprocessor, file_reader, labeler
from tools.classifiers import RfClassifier
from io import StringIO
from pathlib import Path
import shutil

@pytest.fixture(name="get_data", scope="module")
def fixture_get_data():
    infile = StringIO()
    with open(Path.joinpath(pt.TESTS_FILES_DIR, 'screenings.csv'),
              'r', encoding='utf8') as fd:
        shutil.copyfileobj(fd, infile)
        infile.seek(0)
        df = file_reader.read_csv(infile, converters={'Ats': str})
    return df

def test_predict_complete(get_data):
    settings = dict({'alarm_ats': ['222718'],
                     'threshold_weeks': 8,
                     'threshold_training': 10})

    accum_screenings = labeler.accumulate_screenings(get_data, settings)
    df = labeler.make_complete_label(accum_screenings)
    df = preprocessor.split_cat_columns(df, col_to_split='Ats', tag='Ats', resolution=10)

    ats_cols = [str(i)+'Ats' for i in range(1, 11)]
    df_enc = preprocessor.one_hot_encode(df, ats_cols)
    general_cols = df[['Gender', 'BirthYear', 'LoanPeriod', 'NumberAts']]
    df = pd.concat([general_cols, df_enc, df[['Complete']]], axis=1)

    y = df["Complete"]
    X = df.drop(['Complete'], axis=1)

    assert_split(X, y)

    result = RfClassifier(X, y).evaluate(metrics = ['accuracy'], k=0)
    assert result is not None

def assert_split(X, y):
    skut.check_X_y(X, y.values.ravel())
    skut.check_consistent_length(X, y.values.ravel())
