"""
test_neural_embedder.py
====================================
Test module for neural embedder
"""

from tools import file_reader, neural_embedder, preprocessor
from utility.config import load_config
from io import StringIO
from pathlib import Path
import paths as pt
import shutil

def test_fit():
    infile = StringIO()
    settings = load_config(pt.TESTS_FILES_DIR, "test_config.yaml")
    file_path = pt.TESTS_FILES_DIR
    file_name = "complete.csv"
    ats_resolution = settings['ats_resolution']
    converters = {str(i)+'Ats':str for i in range(1, ats_resolution+1)}
    with open(Path.joinpath(file_path, file_name), 'r',
              newline='', encoding='utf8') as fd:
        shutil.copyfileobj(fd, infile)
        infile.seek(0)
        df = file_reader.read_csv(infile, converters=converters)

    emb_cols = df.filter(regex='((\d+)[Ats])\w+', axis=1)
    n_numerical_cols = df.shape[1] - emb_cols.shape[1] - 1
    df_to_enc = df.iloc[:,n_numerical_cols:]

    target_name = "Complete"
    train_ratio = 0.7
    X_train, X_val, y_train, y_val, _ = preprocessor.prepare_data_for_emb(df_to_enc,
                                                                          target_name,
                                                                          train_ratio)
    num_epochs = 10
    network = neural_embedder.NeuralEmbedder(df=df_to_enc,
                                             target_name=target_name,
                                             metrics=['accuracy'],
                                             epochs=num_epochs)
    history = network.fit(X_train, y_train, X_val, y_val)

    assert len(history.history['accuracy']) == num_epochs
