const { PrismaClient } = require('@prisma/client')
const express = require("express");
const bunyan = require('bunyan');
const cors = require("cors");
const axios = require('axios');
var https = require('https');
var http = require('http');
const path = "/vue-app/dist";
const app = express();

app.use(express.static(path));

const prisma = new PrismaClient();

app.use(cors(corsOptions));
app.use(express.json());
app.use(express.urlencoded({ extended: true }));

var corsOptions = {
  origin: "http://localhost:8081"
};

/*var corsOptions = {
   origin: "https://air-app.azurewebsites.net"
};*/

const log = bunyan.createLogger({ name: "AIR_UI_Backend: " });
const predictionURL_training = 'https://air-fastapi.azurewebsites.net/predict_training';
const predictionURL_alarm = 'https://air-fastapi.azurewebsites.net/predict_alarm';

async function deleteVariable(variablesMarkedForDeletion) {
  variablesMarkedForDeletion.forEach(element => {
    element = undefined;
    delete(element);
  });
}

function convertDataSizeToKb(logMessage, data) {
  const datasizeInKb = (new TextEncoder().encode(JSON.stringify(data)).length) / 1024;
  console.log(logMessage, datasizeInKb);
}

function dataPreparationForML(age, gender, assistiveAids, loanPeriod) {
  const Gender = gender == 'Mand' ? 1 : 0;
  const BirthYear = (new Date().getFullYear() - age);
  const LoanPeriod = loanPeriod;
  const Ats = assistiveAids;

  return {
    "Gender": Gender,
    "BirthYear": BirthYear,
    "LoanPeriod": LoanPeriod,
    "Ats": Ats
  }
}

async function getCompleteTrainingPrediction(inputForCompleteTrainingPrediction) {
  predictionResult = axios.post(predictionURL_training, inputForCompleteTrainingPrediction).then(result => {
    return result.data;
  }).catch(error => {
    log.error("getCompleteTrainingPrediction() threw an exception: ", error);
  })
  return predictionResult;
}

async function getAlarmPrediction(inputForAlarmPrediction) {
  predictionResult = axios.post(predictionURL_alarm, inputForAlarmPrediction).then(result => {
    return result.data;
  }).catch(error => {
    log.error("getAlarmPrediction() threw an exception: ", error);
  })
  return predictionResult;
}

async function getCitizenInformation(citizenSSN) {
  const citizen = prisma.citizen.findUnique({
    where: {
      ssn: citizenSSN
    },
    select: {
      first_name: true,
      last_name: true,
      age: true,
      ssn: true,
      gender: true,
      Assistive_Aids: {
        select: {
          name: true,
          iso: true,
          initiative_name: true,
          paragraph: true,
          lend_date: true,
        },
      },
      Training_Plans: {
        select: {
          name: true,
          start_date: true,
          end_date: true,
          status: true,
          time_slot: true,
        },
      },
      Home_Care: {
        select: {
          date: true,
          type: true,
          name: true,
          time_slot: true,
        },
      },
    },
  })
  return citizen
}

app.get('/', function (req, res) {
  res.sendFile(path + "/index.html");
});

app.post("/home/:ssn", async (req, res) => {
  var citizenData = await getCitizenInformation(req.body.citizenSSN);
  if (!citizenData || citizenData.Assistive_Aids.length == 0) {
    await deleteVariable([citizenData]);
    res.send(false);
  } else {

    //Format and Sort Assistive Aids Data from Database
    var assistiveAidsWithCorrectDateFormat = changeLendDateFormat(citizenData.Assistive_Aids);
    var sortedAssistiveAids = sortAssistiveAidsByDate(assistiveAidsWithCorrectDateFormat);

    //Remove invalid assistive aids iso's
    var sortedAndCleanedAssistiveAids = removeInvalidAssistiveAidsFromList(sortedAssistiveAids);

    //Assistive Aids and Loan Period Calculations for the complete training case
    var assistiveAidsTraining = appendAssistiveAidsISO(sortedAndCleanedAssistiveAids);
    var loanPeriod = calculateMeanLoanPeriod(sortedAndCleanedAssistiveAids);

    //Assistive Aids Calculations for the alarm case (it utilizes the same loan period as the complete training case)
    var assistiveAidsWithoutAlarm = removeAlarmFromAssistiveAidsList(sortedAndCleanedAssistiveAids);
    var assistiveAidsAlarm = appendAssistiveAidsISO(assistiveAidsWithoutAlarm);

    //Input to ML model for both cases
    var inputForCompleteTrainingPrediction = dataPreparationForML(citizenData.age, citizenData.gender, assistiveAidsTraining, loanPeriod);
    var inputForAlarmPrediction = dataPreparationForML(citizenData.age, citizenData.gender, assistiveAidsAlarm, loanPeriod);

    //Result from ML model for both cases
    var citizenCompleteTrainingPrediction = await getCompleteTrainingPrediction(inputForCompleteTrainingPrediction);
    var citizenAlarmPrediction = await getAlarmPrediction(inputForAlarmPrediction);
    
    convertDataSizeToKb("Combined datasize from both the ML Model and Database in Kb: ", { citizenData, citizenCompleteTrainingPrediction, citizenAlarmPrediction });
    res.json({ citizenData, citizenCompleteTrainingPrediction, citizenAlarmPrediction });
    
    await deleteVariable([
      citizenData, 
      assistiveAidsWithCorrectDateFormat, 
      sortedAssistiveAids,
      sortedAndCleanedAssistiveAids, 
      assistiveAidsTraining, 
      loanPeriod, 
      inputForCompleteTrainingPrediction,
      inputForAlarmPrediction, 
      citizenCompleteTrainingPrediction, 
      citizenAlarmPrediction
    ]);
  }
});

function changeLendDateFormat(assistiveAids) {
  assistiveAids.forEach(function(assistiveAid) {
    assistiveAid.lend_date = rearrangeDateFormat(assistiveAid.lend_date);
  });
  return assistiveAids;
}

function rearrangeDateFormat(date) {
  var dateInArray = date.split('/');
  var day = dateInArray[0];
  var month = dateInArray[1];
  var year = dateInArray[2];
  var rearrangedDateFormat = month + '/' + day + '/' + '20' + year;
  return rearrangedDateFormat;
}

function sortAssistiveAidsByDate(assistiveAids){
  if(assistiveAids.length == 1) {
    return assistiveAids;
  } else {
  var sortedAssistiveAids = assistiveAids.sort((a, b) => new Date(b.lend_date) - new Date(a.lend_date));
  sortedAssistiveAids.reverse();
  return sortedAssistiveAids;
  }
}

function removeInvalidAssistiveAidsFromList(assistiveAids) {
  for( var i = 0; i < assistiveAids.length; i++) { 
    if ( assistiveAids[i].iso === '999999' || assistiveAids[i].iso === '220681') { 
      assistiveAids.splice(i, 1); 
      i--;
    }
  }
  return assistiveAids;
}

function removeAlarmFromAssistiveAidsList(assistiveAids) {
  for( var i = 0; i < assistiveAids.length; i++) { 
    if ( assistiveAids[i].iso === '222718') { 
      assistiveAids.splice(i, 1); 
      i--;
    }
  }
  return assistiveAids;
}

function appendAssistiveAidsISO(assistiveAids) {
  var assistiveAidsISOString = "";
  for (var i = 0; i < assistiveAids.length; i++) {
    if(i === assistiveAids.length - 1 || i === 9) {
      assistiveAidsISOString += assistiveAids[i].iso;
      break;
    }
    else {
      assistiveAidsISOString += assistiveAids[i].iso + ",";
    }
  } 
  return assistiveAidsISOString;
}

function calculateMeanLoanPeriod(assistiveAids){
  var assistiveAidsLoanPeriod = 0;
  assistiveAids.forEach(function(assistiveAid) {
     assistiveAidsLoanPeriod += dateDifferenceInDays(assistiveAid.lend_date);
  });
  var assistiveAidsMeanLoanPeriod = (assistiveAidsLoanPeriod / assistiveAids.length);
  return Math.round(assistiveAidsMeanLoanPeriod);
}

function dateDifferenceInDays(assistiveAidLendDate) {
  var nowDate = new Date();
  var lendDate = new Date(assistiveAidLendDate);
  var differenceInTime = nowDate.getTime() - lendDate.getTime();
  var oneDayInMilliseconds = 1000*3600*24;
  return Math.round(((differenceInTime))/(oneDayInMilliseconds) - 1);
}

//http.createServer(app).listen(80);
//https.createServer(app).listen(443);
//log.info(`Server is listening on port ${80} and ${443}.`);

const PORT = process.env.PORT || 8080;
 app.listen(PORT, () => {
   log.info(`Server is listening on port ${PORT}.`);
 });